﻿// touchpad-patch.c
// Author: rustyx
// Public Domain

// Installation: Compile release, run install.cmd As Administrator

#define WIN32_LEAN_AND_MEAN
#include <SDKDDKVer.h>
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>

const LPBYTE patchAddress = (LPBYTE) 0x405767;
const BYTE patchOrig[] = {                  0x8B, 0x2D, 0x48, 0xD2, 0x40, 0x00, 0x6A, 0x04, 0xFF,
  0xD5, 0x33, 0xC9, 0x66, 0x85, 0xC0, 0x0F, 0x9C, 0xC1, 0x6A, 0x05, 0x8B, 0xF9, 0xFF, 0xD5, 0x33,
  0xD2, 0x66, 0x85, 0xC0, 0x0F, 0x9C, 0xC2, 0x6A, 0x06, 0x8B, 0xF2, 0xFF, 0xD5, 0x33, 0xDB, 0x66,
  0x85, 0xC0, 0x6A, 0x10, 0x0F, 0x9C, 0xC3, 0xFF, 0xD5, 0x33, 0xC9, 0x66, 0x85, 0xC0, 0x0F, 0x9C,
  0xC1, 0x6A, 0x11, 0x8B, 0xE9, 0xFF, 0x15, 0x48, 0xD2, 0x40, 0x00, 0x33, 0xD2 };
const BYTE patchData[] = {                  0x31, 0xC0, 0x89, 0x44, 0x24, 0x18, 0x89, 0x44, 0x24,
  0x1C, 0x89, 0x44, 0x24, 0x20, 0x89, 0x44, 0x24, 0x2C, 0x89, 0x44, 0x24, 0x30, 0x8B, 0x84, 0x24,
  0xD4, 0x01, 0x00, 0x00, 0xBB, 0x78, 0x00, 0x00, 0x00, 0xF7, 0xEB, 0x89, 0x44, 0x24, 0x24, 0xB8,
  0x00, 0x08, 0x00, 0x00, 0x89, 0x44, 0x24, 0x28, 0x8D, 0x4C, 0x24, 0x18, 0x6A, 0x1C, 0x51, 0x6A,
  0x01, 0xFF, 0x15, 0x24, 0xD2, 0x40, 0x00, 0xE9, 0x9F, 0x00, 0x00, 0x00, 0xCC };

HANDLE hEvent;
void report_event(DWORD id, WORD type)
{
  DWORD data = GetLastError();
  if (!hEvent && !(hEvent = RegisterEventSourceA(NULL, "touchpad-patch")))
    return;
  ReportEventA(hEvent, type, 0, id, NULL, 0, sizeof(data), NULL, &data);
}

void install_patch()
{
  DWORD oldProtect;
  if (!VirtualProtect(patchAddress, sizeof(patchData), PAGE_EXECUTE_READWRITE, &oldProtect)) {
    report_event(2001, EVENTLOG_ERROR_TYPE); return;
  }
  if (memcmp(patchAddress, patchOrig,sizeof(patchOrig))) {
    report_event(2002, EVENTLOG_ERROR_TYPE);
  }
  else {
    memcpy(patchAddress, patchData, sizeof(patchData));
    FlushInstructionCache(GetCurrentProcess(), patchAddress, sizeof(patchData));
    report_event(1000, EVENTLOG_INFORMATION_TYPE);
  }
  VirtualProtect(patchAddress, sizeof(patchData), oldProtect, &oldProtect);
}

const char processName[] = "TouchPad.exe";

BOOL is_interesting_process()
{
  char buf[MAX_PATH];
  DWORD len = GetModuleFileNameA(NULL, buf, MAX_PATH);
  if (len < _countof(processName) - 1)
    return FALSE;
  return _stricmp(buf + len - _countof(processName) + 1, processName) == 0;
}

BOOL APIENTRY DllMain(HMODULE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
  switch (ul_reason_for_call)
  {
  case DLL_PROCESS_ATTACH:
    if (is_interesting_process())
      install_patch();
    break;
  }
  if (hEvent) {
    DeregisterEventSource(hEvent);
  }
  return FALSE;
}
